public class Buch
{
    private String titel = "--undef--";
    private String autor;
    private Verlag verlag;
    private int publJahr = -1;   //Publikations-Jahr
    private String artCode; // eindeutiger Alphanumerischer Artikelcode (z.B. ISBN, ...)

    public Buch(String titel, String autor, Verlag verlag, int publJahr) {
        setTitel(titel);
        setAutor(autor);
        setVerlag(verlag);
        setPublJahr(publJahr);
        //setArtCode(artCode);
    }

    public String getTitel() {
        return titel;
    }

    public void setTitel(String titel) {
        if (titel == null || titel.isBlank()) {
            System.out.format("FEHLER in 'Buch(..)' - ungültiger Titel: '%s' "
                + "- bleibt '%s'%n", titel, this.titel);
        } else {
            this.titel = titel;
        }
    }

    public String getAutor() {
        return autor;
    }

    public void setAutor(String autor) {
        this.autor = autor;
    }

    public Verlag getVerlag() {
        return verlag;
    }

    public void setVerlag(Verlag verlag) {
        if (verlag == null) {
            System.out.format("FEHLER in 'setVerlag(..)' - verlag ist null");
        } else {
            this.verlag = verlag;
            verlag.erfasseBuch(this);  // falls dort noch nicht in Array, wird es hinzugefügt
        }
    }

    public int getPublJahr() {
        return publJahr;
    }

    public void setPublJahr(int publJahr) {
        if (publJahr < 1600) {
            System.out.format("FEHLER in setPublJahr (< 1600) - publJahr = %d - bleibt auf %d%n",
                publJahr, this.publJahr);
        } else {
            this.publJahr = publJahr;
        }
    }

    public String getArtCode() {
        return artCode;
    }

    public void setArtCode(String artCode) {  // dzt. ungenutzt!!!!
        this.artCode = artCode;
    }

    public String autorInitialen() { // aus Hans Jörg Bauer wird H.J.B.
        String[] teile = autor.split(" ");
        String erg = "";
        for (int i = 0; i < teile.length; i++) {
            erg += teile[i].charAt(0) + ".";
        }
        return erg;
    }

    @Override
    public String toString() {
        return String.format("  %s, %d von %s, Verlag %s", titel, publJahr, autor, verlag);
    }

    @Override   // "echtes" equals(...) - funktioniert mit jedem Objekt als Parameter
    public boolean equals(Object obj) {  // wird NICHT für PLÜ benötigt!!!!
        if (obj == null) {
            return false;
        } else if (!(obj instanceof Buch)) {
            return false;
        } else {
            Buch other = (Buch) obj;
            if (!this.titel.equals(other.titel)) {
                return false;
            }
            if (!this.autor.equals(other.autor)) {
                return false;
            }
            if (!this.verlag.equals(other.verlag)) {
                return false;
            }
            if (this.publJahr != other.publJahr) {
                return false;
            }
            return true;
        }
    }

    public boolean equals(Buch other) {   // für uns ausreichendes equals(..) - kann nur Buch-Parameter
        if (other == null) {
            return false;
        } else {
            if (!this.titel.equals(other.titel)) {
                return false;
            }
            if (!this.autor.equals(other.autor)) {
                return false;
            }
            if (!this.verlag.equals(other.verlag)) {
                return false;
            }
            if (this.publJahr != other.publJahr) {
                return false;
            }
            return true;
        }
    }

    public int compareTo(Buch other) {
        int cmp = this.titel.compareTo(other.titel);
        if (cmp == 0) {
            cmp = this.autor.compareTo(other.autor);
        }
        if (cmp == 0) {
            cmp = this.publJahr - other.publJahr;
        }
        if (cmp == 0) {
            cmp = this.verlag.compareTo(other.verlag);
        }
        return cmp;
    }

    public static void testen() {
        // hier sollten analog zu Verlag alle Methoden getestet werden (auch ungültige Fälle!)
        System.out.println("Erzeuge Buch b1 (Faust) ...");
        Buch b1 = new Buch("Faust", "Johann Wolfgang von Goethe", new Verlag("Literatur-Welt"), 1789);
        System.out.println("Erzeuge Buch b2 (Krieg und Frieden) ...");
        Buch b2 = new Buch("Krieg und Frieden", "Lew Nikolajewitsch Graf Tolstoi",
                new Verlag("Literatur-Welt"), 1868);
        System.out.println("Autor-Initialen Faust->Goethe: " + b1.autorInitialen());
        // fortsetzen, sodass im Idealfall alle Methoden und Fehlermöglichkeiten sichtbar werden
        System.out.println("Buch b1:'Faust' ist 'kleiner' als Buch b2:'Krieg und Frieden' "
                + "- b1.compareTo(b2) < 0! - Berechnung: " + b1.compareTo(b2));
        System.out.println("umgekehrt: b2 > b1 -> b2.compareTo(b1) liefert >0: " + b2.compareTo(b1));
        System.out.println("b1 mit sich selbst verglichen: b1.compareTo(b1)==0: " + b1.compareTo(b1));
        System.out.println();
    }
}
