
public class Verlag
{
    private String name = "--undef--";
    private Buch[] buecher;
    private int freeIdx; 

    public Verlag(String name, int kapazitaet) {
        if (name == null || name.isBlank()) {
            System.out.format("FEHLER in 'Verlag(..)' - ungültiger Name: '%s' "
                + "- bleibt '%s'%n", this.name);
        } else {
            this.name = name;
        }
        buecher = new Buch[kapazitaet];
        freeIdx = 0;
    }

    public Verlag(String name) {
        this(name, 10);   // Aufruf des anderen Konstruktors, Kapazität auf 10 gesetzt
    }    

    public boolean erfasseBuch(Buch buch) {  // bei erstmaligeen Aufruf mit gegebenem Buch wird dieses
        boolean neuRegist = false;           //.. in die Bücherliste des Verlags aufgenommen.
        if (buch == null) {
            System.out.format("FEHLER in 'erfasseBuch(..)' - buch 'null'%n");
        } else if (!verlagPasst(buch)) {
            System.out.format("FEHLER in 'erfasseBuch(..)' - falscher Verlag: '%s'%n", buch);
        } else if (schonErfasst(buch)) {
            System.out.format("HINWEIS - Buch schon zuvor erfasst: '%s'%n", buch);
        } else {
            System.out.format("Neues Buch wird in Verlag erfasst: '%s'%n", buch);
            buecher[freeIdx] = buch;
            freeIdx++;
            neuRegist = true;
        }
        return neuRegist;
    }

    public boolean verlagPasst(Buch buch) {
        if (!buch.getVerlag().equals(this)) {
            System.out.format("FEHLER in 'verlagPasst(..)' - aktueller Verlag '%s',"
                + " Buch-Verlag: '%s'%n", this, buch.getVerlag());
            return false;
        }
        return true;
    }

    public boolean schonErfasst(Buch buch) {
        for (int i = 0; i < freeIdx; i++) {
            if (buch.equals(buecher[i])) {
                return true;
            }
        }
        return false;
    }

    public String toString() {
        return String.format("%s, Bücherzahl=%d", name, freeIdx);
    }

    public boolean equals(Verlag other) {
        if (other == null) {
            return false;
        } else if (!this.name.equals(other.name)) {
            return false;
        }
        return true;
    }

    public int compareTo(Verlag other) {  // Verlag Attribut v. Buch, wir f. Vergl. v. Buch genutzt
        return this.name.compareTo(other.name);
    }

    public int findMinValPos(int anfIdx, int endIdxExcl) {
        if (anfIdx >= endIdxExcl || endIdxExcl > freeIdx) { // nötig: Anfang<Ende, Ende max freeIdx 
            return -1;
        }  // Prüfung ^ kann für Minimalversion weggelassen werden
        // ab hier Kern-Algorithmus:
        int minValPos = anfIdx;  // provisorisch minValPos auf anfIdx setzen
        for (int i = anfIdx + 1; i < freeIdx; i++) {
            if (buecher[i].compareTo(buecher[minValPos]) < 0) { // <-- compareTo(..) von Buecher!
                minValPos = i;
            }
        }
        return minValPos;
    }

    public void tauschen(int idx0, int idx1) {
        if (idx0 != idx1) {  // wenn beide gleich: Tausch mit sich selbst - kann man sich ersparen!
            Buch tmp = buecher[idx0];   // Dreieckstausch:
            buecher[idx0] = buecher[idx1];
            buecher[idx1] = tmp;
        }
    }

    public void selectionSort() {
        for (int i = 0; i < freeIdx; i++) {              // NUR bis freeIdx!!!
            int minValIdx = findMinValPos(i, freeIdx);  // finden der Position des kleinsten Wertes
            tauschen(i, minValIdx);                    // tauschen m. vorderstem Elem. dieser Runde
        }
    }

    public void buecherSortieren() {
        selectionSort();   // da könnte man alternativ z.B. einen Bubble-Sort aufrufen
    }

    public void printBuecher(String ueberschrift) {
        String header = String.format("===== %s =====", ueberschrift); // %s ersetzt durch ''
        System.out.println(header);
        for (int i = 0; i < freeIdx; i++) {
            System.out.format("  %s%n", buecher[i]);
        }
    }

    public void printBuecherWoTitelEnthaelt(String teilstueck) {
        for (int i = 0; i < freeIdx; i++) {
            if (buecher[i].getTitel().contains(teilstueck)) {
                System.out.println(buecher[i]);
            }
        }
    }

    // Statische Test-Methode - in BlueJ bei Rechtsklick auf Klasse direkt unter Konstruktoren aufrufbar!
    public static void testen() {
        System.out.println();
        Verlag v1 = new Verlag("Demo-Verlag", 100);
        System.out.println("Neues Buch erzeugen: Winnetou 1 ...");
        Buch winnetou1 = new Buch("Winnetou 1", "Karl May", v1, 1878);
        System.out.println("Neues Buch erzeugen: Winnetou 2 ...");
        Buch winnetou2 = new Buch("Winnetou 2", "Karl May", v1, 1880);
        System.out.println("Neues Buch erzeugen: Winnetou 3 ...");
        Buch winnetou3 = new Buch("Winnetou 3", "Karl May", v1, 1884);
        System.out.println("Neues Buch erzeugen: Krieg und Frieden ...");
        Buch kriegUndFrieden = new Buch("Krieg und Frieden", "Lew Nikolajewitsch Graf Tolstoi", v1, 1868);
        System.out.println("Neues Buch erzeugen: Dies und das ...");
        Buch diesUndDas = new Buch("Dies und das", "Irgend Wer", v1, 1999);
        System.out.println("v1.erfasseBuch(winnetou2) ...");
        v1.erfasseBuch(winnetou2);
        System.out.println("v1.erfasseBuch(winnetou2) - Hinweis erwartet! ...");
        v1.erfasseBuch(winnetou2);  // Versuch, doppelt zu erfassen
        System.out.println("v1.erfasseBuch(null) - Fehler erwartet! ...");
        v1.erfasseBuch(null);       // Fehler provozieren
        System.out.println("v1.erfasseBuch(kriegUndFrieden) ...");
        v1.erfasseBuch(kriegUndFrieden);
        System.out.println("v1.erfasseBuch(diesUndDas) ...");
        v1.erfasseBuch(diesUndDas);  // regulär
        System.out.println("v1.erfasseBuch(winnetou3) ...");
        v1.erfasseBuch(winnetou3);  // regulär
        System.out.println("v1.erfasseBuch(winnetou1) ...");
        v1.erfasseBuch(winnetou1);  // regulär
        System.out.println("sortieren ...");
        v1.printBuecher("Kontroll-Ausgabe VOR Sortierung");
        v1.buecherSortieren();
        v1.printBuecher("Kontroll-Ausgabe NACH Sortierung");
        System.out.println("printBuecherWoTitelEnthaelt()");
        v1.printBuecherWoTitelEnthaelt("und");
        // fortsetzen, sodass im Idealfall alle Methoden und Fehlermöglichkeiten sichtbar werden
    }
}
